import jas.probe.*;
import jas.engine.*;
import jas.stats.*;
import jas.events.*;
import jas.io.*;
import jas.space.*;
import jas.plot.*;

import jas.net.*;

import java.util.*;
import java.awt.Color;
import java.io.File;

/**
 * A translation of the Swarm example simulation Heat Bugs. The heat bugs
 * simulation consists of heat bugs - simple agents that absorb and expel heat
 * and a heatspace which diffuses this heat into the area surrounding the
 * bug. Heat bugs have an ideal temperature and will move about the space
 * in attempt to achieve this idea temperature.
 *
 * @author Swarm Project and Michele Sonnessa
 */

public class HeatBugsSrvModel extends SimModel
{

  public int numBugs = 100;
  public double evapRate = 0.99;
  public double diffusionConstant = 1.0;
  public int worldXSize = 80;
  public int worldYSize = 80;
  public int minIdealTemp = 17000;
  public int maxIdealTemp = 31000;
  public int minOutputHeat = 3000;
  public int maxOutputHeat = 10000;
  public float randomMoveProbability = 0.0f;

  public int steps = 1000;

  public AgentList heatBugList;
  public Grid world;
  public HeatSpace space;

  public SequenceStats unhapSeq;
  public TimeSeries unhapTime;

  InformationServer server;

  public void setParameters()
  {
    ParametersBag bag = new ParametersBag(path + "HeatbugsParams.pForm.xml");
    ParameterFrame fr = new ParameterFrame(this, "Heatbugs", bag);
    addSimWindow(fr);

  }

  public void buildModel()
  {
    buildObjects();
    buildActions();
  }

  private void buildObjects()
  {
	heatBugList = new AgentList();
    space = new HeatSpace(diffusionConstant, evapRate, worldXSize, worldYSize);
    world = new ObjGrid(worldXSize, worldYSize);

    for (int i = 0; i < numBugs; i++) {
      int idealTemp = Sim.getRnd().getIntFromTo(minIdealTemp, maxIdealTemp);
      int outputHeat = Sim.getRnd().getIntFromTo(minOutputHeat, maxOutputHeat);
      int x, y;

      do {
        x = Sim.getRnd().getIntFromTo(0, space.getXSize() - 1);
        y = Sim.getRnd().getIntFromTo(0, space.getYSize() - 1);
      } while (world.get(x, y) != null);

      HeatBug bug = new HeatBug(i, space, (IObjGrid) world, x, y, idealTemp, outputHeat,
                                randomMoveProbability, this);
      world.set(x, y, bug);
      heatBugList.add(bug);
    }

    unhapSeq = new SequenceStats("unhappiness", heatBugList, HeatBug.class, "unhappiness", false);
    unhapTime = new TimeSeries();
    unhapTime.addSeries("mean", unhapSeq, SequenceStats.F_MEAN);
    unhapTime.addSeries("min", unhapSeq, SequenceStats.F_MIN);
    unhapTime.addSeries("max", unhapSeq, SequenceStats.F_MAX);

  }

  public void regeneratePreferences()
  {
	for (int i = 0; i < heatBugList.size(); i++)
	{
		HeatBug bug = (HeatBug) heatBugList.get(i);
		int idealTemp = Sim.getRnd().getIntFromTo(minIdealTemp, maxIdealTemp);
		bug.setIdealTemp(idealTemp);
	}

  }

  private void buildActions()
  {
    SimGroupEvent s;

    s = eventList.scheduleGroup(0, 1);
    s.addEvent(space, "diffuse");
    s.addEvent(heatBugList, HeatBug.EVENT_STEP);
    s.addEvent(space, Sim.EVENT_UPDATE);
    s.addEvent(unhapSeq, Sim.EVENT_UPDATE);
    s.addEvent(unhapTime, Sim.EVENT_UPDATE);

    eventList.scheduleSystem(steps, Sim.EVENT_SHUTDOWN);

  }

  public static void main(String[] args)
  {

    HeatBugsSrvModel p = new HeatBugsSrvModel();
    SimEngine eng = new SimEngine();

    eng.addModel(p);
    eng.startGUI(false);

  }

}
